package explorviz.live_trace_processing.filter.reconstruction;

import java.util.ArrayList;
import java.util.List;

import explorviz.live_trace_processing.Constants;
import explorviz.live_trace_processing.record.event.AbstractOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.AfterFailedOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.AfterOperationEventRecord;
import explorviz.live_trace_processing.record.event.normal.BeforeOperationEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

class TraceReconstructionBuffer {
	private final List<AbstractOperationEventRecord> events = new ArrayList<AbstractOperationEventRecord>(
			Constants.TRACE_RECONSTRUCTION_BUFFER_INITIAL_SIZE);

	private boolean closeable;

	private int openEvents;

	private long maxLoggingTimestamp = -1;
	private int maxOrderIndex = -1;

	public final void insertEvent(final AbstractOperationEventRecord event) {
		setMaxLoggingTimestamp(event);
		final int orderIndex = setMaxOrderIndex(event);

		if (event instanceof BeforeOperationEventRecord) {
			if (orderIndex == 0) {
				closeable = true;
			}
			openEvents++;
		} else if (event instanceof AfterOperationEventRecord) {
			openEvents--;
		} else if (event instanceof AfterFailedOperationEventRecord) {
			openEvents--;
		}

		events.add(event);
	}

	public final long getMaxLoggingTimestamp() {
		return maxLoggingTimestamp;
	}

	private final void setMaxLoggingTimestamp(final AbstractOperationEventRecord event) {
		final long loggingTimestamp = event.getLoggingTimestamp();
		if (loggingTimestamp > maxLoggingTimestamp) {
			maxLoggingTimestamp = loggingTimestamp;
		}
	}

	private final int setMaxOrderIndex(final AbstractOperationEventRecord event) {
		final int orderIndex = event.getOrderIndex();
		if (orderIndex > maxOrderIndex) {
			maxOrderIndex = orderIndex;
		}
		return orderIndex;
	}

	public final boolean isFinished() {
		return !isInvalid();
	}

	public final boolean isInvalid() {
		return ((openEvents != 0) || events.isEmpty() || ((maxOrderIndex + 1) != events.size()) || !closeable);
	}

	public final Trace toTrace(final boolean valid) {
		return new Trace(events, valid);
	}
}
